@@ -1,3 +1,5 @@
+import warnings
+
 import h5py
 import cupy as cp
 from pygpe.shared.grid import Grid
@@ -38,24 +40,24 @@ class DataManager:
         :param parameters: The parameter dictionary.
         :type parameters: dict
         """
+        if grid.ndim > 3 or grid.ndim < 1:
+            raise ValueError(f"Grid with dimension of {grid.ndim} is unsupported, please use a 1D, 2D, or 3D grid.")
         self._save_grid_params(grid)
         self._save_initial_wfn(wfn)
         self._save_params(parameters)
 
     def _save_grid_params(self, grid: Grid) -> None:
         """Saves grid parameters to dataset."""
-        if grid.ndim == 1:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+        with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+            if grid.ndim == 1:
                 file.create_dataset(dmp.GRID_NX, data=grid.num_points_x)
                 file.create_dataset(dmp.GRID_DX, data=grid.grid_spacing_x)
-        elif grid.ndim == 2:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+            elif grid.ndim == 2:
                 file.create_dataset(dmp.GRID_NX, data=grid.num_points_x)
                 file.create_dataset(dmp.GRID_NY, data=grid.num_points_y)
                 file.create_dataset(dmp.GRID_DX, data=grid.grid_spacing_x)
                 file.create_dataset(dmp.GRID_DY, data=grid.grid_spacing_y)
-        elif grid.ndim == 3:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+            elif grid.ndim == 3:
                 file.create_dataset(dmp.GRID_NX, data=grid.num_points_x)
                 file.create_dataset(dmp.GRID_NY, data=grid.num_points_y)
                 file.create_dataset(dmp.GRID_NZ, data=grid.num_points_z)
@@ -65,16 +67,15 @@ class DataManager:
 
     def _save_initial_wfn(self, wfn: Wavefunction) -> None:
         """Saves initial wavefunction to dataset."""
-        if wfn.grid.ndim == 1:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+        with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+            if wfn.grid.ndim == 1:
                 file.create_dataset(
                     dmp.SCALAR_WAVEFUNCTION,
                     (wfn.grid.shape, 1),
                     maxshape=(wfn.grid.shape, None),
                     dtype="complex128",
                 )
-        else:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+            else:
                 file.create_dataset(
                     dmp.SCALAR_WAVEFUNCTION,
                     (*wfn.grid.shape, 1),
@@ -95,13 +96,12 @@ class DataManager:
         :type wfn: :class:`Wavefunction`
         """
         wfn.ifft()  # Update real-space wavefunction before saving
-        if wfn.grid.ndim == 1:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as data:
+        with h5py.File(f"{self.data_path}/{self.filename}", "r+") as data:
+            if wfn.grid.ndim == 1:
                 new_psi = data[dmp.SCALAR_WAVEFUNCTION]
                 new_psi.resize((wfn.grid.num_points_x, self._time_index + 1))
                 new_psi[:, self._time_index] = cp.asnumpy(wfn.component)
-        else:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as data:
+            else:
                 new_psi = data[dmp.SCALAR_WAVEFUNCTION]
                 new_psi.resize((*wfn.grid.shape, self._time_index + 1))
                 new_psi[..., self._time_index] = cp.asnumpy(wfn.component)
