@@ -37,6 +37,8 @@ class DataManager:
         :param parameters: The parameter dictionary.
         :type parameters: dict
         """
+        if grid.ndim > 3 or grid.ndim < 1:
+            raise ValueError(f"Grid with dimension of {grid.ndim} is unsupported, please use a 1D, 2D, or 3D grid.")
         self._save_initial_grid_params(grid)
         self._save_initial_wfn(wfn)
         self._save_params(parameters)
@@ -45,18 +47,16 @@ class DataManager:
         """Creates new datasets in file for grid-related parameters and saves
         initial values.
         """
-        if grid.ndim == 1:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+        with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+            if grid.ndim == 1:
                 file.create_dataset(dmp.GRID_NX, data=grid.num_points_x)
                 file.create_dataset(dmp.GRID_DX, data=grid.grid_spacing_x)
-        elif grid.ndim == 2:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+            elif grid.ndim == 2:
                 file.create_dataset(dmp.GRID_NX, data=grid.num_points_x)
                 file.create_dataset(dmp.GRID_NY, data=grid.num_points_y)
                 file.create_dataset(dmp.GRID_DX, data=grid.grid_spacing_x)
                 file.create_dataset(dmp.GRID_DY, data=grid.grid_spacing_y)
-        elif grid.ndim == 3:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+            elif grid.ndim == 3:
                 file.create_dataset(dmp.GRID_NX, data=grid.num_points_x)
                 file.create_dataset(dmp.GRID_NY, data=grid.num_points_y)
                 file.create_dataset(dmp.GRID_NZ, data=grid.num_points_z)
@@ -68,8 +68,8 @@ class DataManager:
         """Creates new datasets in file for the wavefunction and saves
         initial values.
         """
-        if wfn.grid.ndim == 1:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+        with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+            if wfn.grid.ndim == 1:
                 file.create_dataset(
                     dmp.SPIN1_WAVEFUNCTION_PLUS,
                     (wfn.grid.shape, 1),
@@ -88,8 +88,7 @@ class DataManager:
                     maxshape=(wfn.grid.shape, None),
                     dtype="complex128",
                 )
-        else:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as file:
+            else:
                 file.create_dataset(
                     dmp.SPIN1_WAVEFUNCTION_PLUS,
                     (*wfn.grid.shape, 1),
@@ -124,8 +123,8 @@ class DataManager:
         :type wfn: :class:`Wavefunction`
         """
         wfn.ifft()  # Update real-space wavefunction before saving
-        if wfn.grid.ndim == 1:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as data:
+        with h5py.File(f"{self.data_path}/{self.filename}", "r+") as data:
+            if wfn.grid.ndim == 1:
                 new_psi_plus = data[dmp.SPIN1_WAVEFUNCTION_PLUS]
                 new_psi_plus.resize((wfn.grid.num_points_x, self._time_index + 1))
                 new_psi_plus[:, self._time_index] = cp.asnumpy(wfn.plus_component)
@@ -137,8 +136,7 @@ class DataManager:
                 new_psi_minus = data[dmp.SPIN1_WAVEFUNCTION_MINUS]
                 new_psi_minus.resize((wfn.grid.num_points_x, self._time_index + 1))
                 new_psi_minus[:, self._time_index] = cp.asnumpy(wfn.minus_component)
-        else:
-            with h5py.File(f"{self.data_path}/{self.filename}", "r+") as data:
+            else:
                 new_psi_plus = data[dmp.SPIN1_WAVEFUNCTION_PLUS]
                 new_psi_plus.resize((*wfn.grid.shape, self._time_index + 1))
                 new_psi_plus[..., self._time_index] = cp.asnumpy(wfn.plus_component)
