@@ -71,12 +71,12 @@ class Wavefunction:
         self._update_atom_numbers()
 
     def set_custom_components(
-        self,
-        plus2_component: cp.ndarray = None,
-        plus1_component: cp.ndarray = None,
-        zero_component: cp.ndarray = None,
-        minus1_component: cp.ndarray = None,
-        minus2_component: cp.ndarray = None,
+            self,
+            plus2_component: cp.ndarray = None,
+            plus1_component: cp.ndarray = None,
+            zero_component: cp.ndarray = None,
+            minus1_component: cp.ndarray = None,
+            minus2_component: cp.ndarray = None,
     ) -> None:
         """Sets the wavefunction components to the specified arrays.
 
@@ -98,39 +98,46 @@ class Wavefunction:
             self.minus2_component = minus2_component
 
     def add_noise_to_components(
-        self, components: str, mean: float, std_dev: float
+            self, components: str | list[str], mean: float, std_dev: float
     ) -> None:
         """Adds noise to the specified wavefunction components
         using a normal distribution.
 
-        :param components: "outer", "inner", "outer+inner", "middle", or "all": The components to add noise to.
+        :param components: "all", "plus2", "plus1", "zero", "minus1", "minus2"
+            or list of strings specifying the components to add noise to.
         :param mean: The mean of the normal distribution.
         :param std_dev: The standard deviation of the normal distribution.
         """
-        if components.lower() == "outer":
-            self.plus2_component += self._generate_complex_normal_dist(mean, std_dev)
-            self.minus2_component += self._generate_complex_normal_dist(mean, std_dev)
-        elif components.lower() == "inner":
-            self.plus1_component += self._generate_complex_normal_dist(mean, std_dev)
-            self.minus1_component += self._generate_complex_normal_dist(mean, std_dev)
-        elif components.lower() == "outer+inner":
-            self.plus2_component += self._generate_complex_normal_dist(mean, std_dev)
-            self.plus1_component += self._generate_complex_normal_dist(mean, std_dev)
-            self.minus1_component += self._generate_complex_normal_dist(mean, std_dev)
-            self.minus2_component += self._generate_complex_normal_dist(mean, std_dev)
-        elif components.lower() == "middle":
-            self.zero_component += self._generate_complex_normal_dist(mean, std_dev)
-        elif components.lower() == "all":
-            self.plus2_component += self._generate_complex_normal_dist(mean, std_dev)
-            self.plus1_component += self._generate_complex_normal_dist(mean, std_dev)
-            self.zero_component += self._generate_complex_normal_dist(mean, std_dev)
-            self.minus1_component += self._generate_complex_normal_dist(mean, std_dev)
-            self.minus2_component += self._generate_complex_normal_dist(mean, std_dev)
-        else:
-            raise ValueError(f"{components} is not a supported configuration")
+        match components:
+            case [*_]:
+                for component in components:
+                    self._add_noise_to_components(component, mean, std_dev)
+            case "all":
+                for component in ["plus2", "plus1", "zero", "minus1", "minus2"]:
+                    self._add_noise_to_components(component, mean, std_dev)
+            case str(component):
+                self._add_noise_to_components(component, mean, std_dev)
+            case _:
+                raise ValueError(f"{components} is not a supported configuration")
 
         self._update_atom_numbers()
 
+    def _add_noise_to_components(self, component: str, mean: float, std_dev: float) -> None:
+        """Adds noise from drawn from a normal distribution to the specified component."""
+        match component.lower():
+            case "plus2":
+                self.plus2_component += self._generate_complex_normal_dist(mean, std_dev)
+            case "plus1":
+                self.plus1_component += self._generate_complex_normal_dist(mean, std_dev)
+            case "zero":
+                self.zero_component += self._generate_complex_normal_dist(mean, std_dev)
+            case "minus1":
+                self.minus1_component += self._generate_complex_normal_dist(mean, std_dev)
+            case "minus2":
+                self.minus2_component += self._generate_complex_normal_dist(mean, std_dev)
+            case _:
+                raise ValueError(f"{component} is not a supported configuration")
+
     def _generate_complex_normal_dist(self, mean: float, std_dev: float) -> cp.ndarray:
         """Returns a ndarray of complex values containing results from
         a normal distribution.
@@ -140,7 +147,7 @@ class Wavefunction:
         ) + 1j * cp.random.normal(mean, std_dev, size=self.grid.shape)
 
     def apply_phase(
-        self, phase: cp.ndarray, components: str | list[str] = "all"
+            self, phase: cp.ndarray, components: str | list[str] = "all"
     ) -> None:
         """Applies a phase to specified components.
 
@@ -148,32 +155,36 @@ class Wavefunction:
         :param components: "all", "plus2", "plus1", "zero", "minus1", "minus2" or a list of strings specifying the
             required components.
         """
-        if isinstance(components, list):
-            for component in components:
+        match components:
+            case [*_]:
+                for component in components:
+                    self._apply_phase_to_component(phase, component)
+            case "all":
+                for component in ["plus2", "plus1", "zero", "minus1", "minus2"]:
+                    self._apply_phase_to_component(phase, component)
+            case str(component):
                 self._apply_phase_to_component(phase, component)
-        elif components == "all":
-            for component in ["plus2", "plus1", "zero", "minus1", "minus2"]:
-                self._apply_phase_to_component(phase, component)
-        elif isinstance(components, str):
-            self._apply_phase_to_component(phase, components)
-        else:
-            raise ValueError(f"Components type {components} is unsupported")
+            case _:
+                raise ValueError(f"Components type {components} is unsupported")
 
-    def _apply_phase_to_component(self, phase: cp.ndarray, component: str):
-        if component == "plus2":
-            self.plus2_component *= cp.exp(1j * phase)
-        elif component == "plus1":
-            self.plus1_component *= cp.exp(1j * phase)
-        elif component == "zero":
-            self.zero_component *= cp.exp(1j * phase)
-        elif component == "minus1":
-            self.minus1_component *= cp.exp(1j * phase)
-        elif component == "minus2":
-            self.minus1_component *= cp.exp(1j * phase)
-        else:
-            raise ValueError(f"Component type {component} is unsupported")
+    def _apply_phase_to_component(self, phase: cp.ndarray, component: str) -> None:
+        """Applies the specified phase to the specified component."""
+        match component.lower():
+            case "plus2":
+                self.plus2_component *= cp.exp(1j * phase)
+            case "plus1":
+                self.plus1_component *= cp.exp(1j * phase)
+            case "zero":
+                self.zero_component *= cp.exp(1j * phase)
+            case "minus1":
+                self.minus1_component *= cp.exp(1j * phase)
+            case "minus2":
+                self.minus2_component *= cp.exp(1j * phase)
+            case _:
+                raise ValueError(f"Component type {component} is unsupported")
 
-    def _update_atom_numbers(self):
+    def _update_atom_numbers(self) -> None:
+        """Calculates and updates the atom numbers for each component."""
         self.atom_num_plus2 = self.grid.grid_spacing_product * cp.sum(
             cp.abs(self.plus2_component) ** 2
         )
@@ -190,7 +201,7 @@ class Wavefunction:
             cp.abs(self.minus2_component) ** 2
         )
 
-    def fft(self):
+    def fft(self) -> None:
         """Fourier transforms real-space components and updates Fourier-space components."""
         self.fourier_plus2_component = cp.fft.fftn(self.plus2_component)
         self.fourier_plus1_component = cp.fft.fftn(self.plus1_component)
@@ -198,7 +209,7 @@ class Wavefunction:
         self.fourier_minus1_component = cp.fft.fftn(self.minus1_component)
         self.fourier_minus2_component = cp.fft.fftn(self.minus2_component)
 
-    def ifft(self):
+    def ifft(self) -> None:
         """Inverse Fourier transforms Fourier-space components and updates real-space components."""
         self.plus2_component = cp.fft.ifftn(self.fourier_plus2_component)
         self.plus1_component = cp.fft.ifftn(self.fourier_plus1_component)
@@ -207,7 +218,7 @@ class Wavefunction:
         self.minus2_component = cp.fft.ifftn(self.fourier_minus2_component)
 
 
-def _uniaxial_initial_state(wfn: Wavefunction, params: dict):
+def _uniaxial_initial_state(wfn: Wavefunction, params: dict) -> None:
     """Sets wavefunction components to uniaxial nematic state."""
     wfn.plus2_component = cp.zeros(wfn.grid.shape, dtype="complex128")
     wfn.plus1_component = cp.zeros(wfn.grid.shape, dtype="complex128")
@@ -218,24 +229,24 @@ def _uniaxial_initial_state(wfn: Wavefunction, params: dict):
     wfn.minus2_component = cp.zeros(wfn.grid.shape, dtype="complex128")
 
 
-def _biaxial_initial_state(wfn: Wavefunction, params: dict):
+def _biaxial_initial_state(wfn: Wavefunction, params: dict) -> None:
     """Sets wavefunction components to biaxial nematic polar state."""
     wfn.plus2_component = (
-        cp.sqrt(params["n0"])
-        / cp.sqrt(2.0)
-        * cp.ones(wfn.grid.shape, dtype="complex128")
+            cp.sqrt(params["n0"])
+            / cp.sqrt(2.0)
+            * cp.ones(wfn.grid.shape, dtype="complex128")
     )
     wfn.plus1_component = cp.zeros(wfn.grid.shape, dtype="complex128")
     wfn.zero_component = cp.zeros(wfn.grid.shape, dtype="complex128")
     wfn.minus1_component = cp.zeros(wfn.grid.shape, dtype="complex128")
     wfn.minus2_component = (
-        cp.sqrt(params["n0"])
-        / cp.sqrt(2.0)
-        * cp.ones(wfn.grid.shape, dtype="complex128")
+            cp.sqrt(params["n0"])
+            / cp.sqrt(2.0)
+            * cp.ones(wfn.grid.shape, dtype="complex128")
     )
 
 
-def _ferromagnetic2p_initial_state(wfn: Wavefunction, params: dict):
+def _ferromagnetic2p_initial_state(wfn: Wavefunction, params: dict) -> None:
     """Sets wavefunction components to ferromagnetic (F=2) state, with atoms in the plus two component."""
     wfn.plus2_component = cp.sqrt(params["n0"]) * cp.ones(
         wfn.grid.shape, dtype="complex128"
@@ -246,7 +257,7 @@ def _ferromagnetic2p_initial_state(wfn: Wavefunction, params: dict):
     wfn.minus2_component = cp.zeros(wfn.grid.shape, dtype="complex128")
 
 
-def _ferromagnetic2m_initial_state(wfn: Wavefunction, params: dict):
+def _ferromagnetic2m_initial_state(wfn: Wavefunction, params: dict) -> None:
     """Sets wavefunction components to ferromagnetic (F=2) state, with atoms in the minus two component."""
     wfn.plus2_component = cp.zeros(wfn.grid.shape, dtype="complex128")
     wfn.plus1_component = cp.zeros(wfn.grid.shape, dtype="complex128")
@@ -257,7 +268,7 @@ def _ferromagnetic2m_initial_state(wfn: Wavefunction, params: dict):
     )
 
 
-def _ferromagnetic1p_initial_state(wfn: Wavefunction, params: dict):
+def _ferromagnetic1p_initial_state(wfn: Wavefunction, params: dict) -> None:
     """Sets wavefunction components to ferromagnetic (F=1) state, with atoms in the plus one component."""
     wfn.plus2_component = cp.zeros(wfn.grid.shape, dtype="complex128")
     wfn.plus1_component = cp.sqrt(params["n0"]) * cp.ones(
@@ -268,7 +279,7 @@ def _ferromagnetic1p_initial_state(wfn: Wavefunction, params: dict):
     wfn.minus2_component = cp.zeros(wfn.grid.shape, dtype="complex128")
 
 
-def _ferromagnetic1m_initial_state(wfn: Wavefunction, params: dict):
+def _ferromagnetic1m_initial_state(wfn: Wavefunction, params: dict) -> None:
     """Sets wavefunction components to ferromagnetic (F=1) state, with atoms in the minus one component."""
     wfn.plus2_component = cp.zeros(wfn.grid.shape, dtype="complex128")
     wfn.plus1_component = cp.zeros(wfn.grid.shape, dtype="complex128")
@@ -279,21 +290,21 @@ def _ferromagnetic1m_initial_state(wfn: Wavefunction, params: dict):
     wfn.minus2_component = cp.zeros(wfn.grid.shape, dtype="complex128")
 
 
-def _cyclic_initial_state(wfn: Wavefunction, params: dict):
+def _cyclic_initial_state(wfn: Wavefunction, params: dict) -> None:
     """Sets wavefunction components to (two-component) cyclic state,
     with atoms in the plus two and minus one components."""
     fz = params["p"] + params["q"] / (params["c2"] * params["n0"])
 
     wfn.plus2_component = (
-        cp.sqrt(params["n0"])
-        * cp.sqrt((1 + fz) / 3)
-        * cp.ones(wfn.grid.shape, dtype="complex128")
+            cp.sqrt(params["n0"])
+            * cp.sqrt((1 + fz) / 3)
+            * cp.ones(wfn.grid.shape, dtype="complex128")
     )
     wfn.plus1_component = cp.zeros(wfn.grid.shape, dtype="complex128")
     wfn.zero_component = cp.zeros(wfn.grid.shape, dtype="complex128")
     wfn.minus1_component = (
-        cp.sqrt(params["n0"])
-        * cp.sqrt((2 - fz) / 3)
-        * cp.ones(wfn.grid.shape, dtype="complex128")
+            cp.sqrt(params["n0"])
+            * cp.sqrt((2 - fz) / 3)
+            * cp.ones(wfn.grid.shape, dtype="complex128")
     )
     wfn.minus2_component = cp.zeros(wfn.grid.shape, dtype="complex128")
